var CaseSummary = function()
{
	var cs = {};
	var debug = 1;
	var TRUNCATE_SIZE = [-1, 30, 20, 15];

	cs.initPage = function()
	{
		Events.addEventHandler('showConfidential', 'change', cs.showConfidentialOnChange);
		Events.doOnEnter('caseName', cs.caseSummary);
		Buttons.addClickHandler('btnCaseSummary', cs.caseSummary);
		Buttons.addClickHandler('printButton', cs.goPrint);
		Buttons.addClickHandler('close', App.closeWindow);
		
		var caseName = Data.get('page-data', 'case-name');
		var isAdmin = Data.int('page-data', 'admin');
		var isPatientCurator = Data.int('page-data', 'patient-curator');

		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName='+encodeURIComponent(caseName);
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, false);
		request.send(null);

		if (debug) App.debug(request.responseText);
		
		var response = JSON.parse(request.responseText);
		if (response.status != 'ok')
		{
			setFatalError(response.message);
			return false;
		}
		
		Main.show('all-info');
		
		var caseInfo = response.caseInfo;
		var site = response.site;
		cs.initSections(response.sections);
		var consentOk = true;

		if (caseInfo)
		{
			// Use returned case name, as it may have been found from personal number
			caseName = caseInfo.name;
			Data.set('page-data', 'case-name', caseName);
			// Update header text to use the potentially changed case name
			if (Doc.element('path-case-summary'))
			{
				Doc.element('path-case-summary').innerHTML = 'Case summary - ' + Strings.encodeTags(caseName);
			}
			consentOk = !caseInfo.consent || caseInfo.consent == 'Yes';
			cs.addColumn('case.name', cs.asLink('SAMPLE', caseInfo));
			cs.addColumn('case.registrationDate', cs.asDate(caseInfo.registrationDate));
			cs.addColumn('case.site', site ? site.name : null);
			cs.addColumn('case.consent', cs.asConsent(caseInfo.consent, cs.asDate(caseInfo.consentDate)));
			cs.addColumn('case.laterality', consentOk ? cs.warnIfMissing(caseInfo.laterality) : null);
			cs.addColumn('case.comment', caseInfo.comment);
		}
		else
		{
			cs.addColumn('case.name', cs.asNoInfo('No case information has been registered'));
			Doc.hide('case-details');
		}

		var patient = response.patient;
		if (patient)
		{
			if (consentOk)
			{
				cs.addColumn('patient.name', cs.asLink('BIOSOURCE', patient));
				cs.addColumn('patient.registrationDate', cs.asDate(patient.registrationDate));
				cs.addColumn('patient.personalNumber', cs.warnIfMissing(patient.personalNumber));
				cs.addColumn('patient.allFirstNames', cs.warnIfMissing(patient.allFirstNames));
				cs.addColumn('patient.familyName', cs.warnIfMissing(patient.familyName));
				cs.addColumn('patient.gender', patient.gender);
				cs.addColumn('patient.allCases', cs.asCaseList(patient.allCases, caseName));
			}
			else
			{
				Doc.hide('patient-info');
			}
		}
		else
		{
			cs.addColumn('patient.name', cs.asNoInfo('No patient information has been registered'));
			Doc.hide('patient-details');
		}

		var blood = consentOk ? response.blood : null;
		if (blood && blood.length > 0)
		{
			for (var i = 0; i < blood.length; i++)
			{
				var b = blood[i];
				cs.addColumn('blood.name', cs.asLink('SAMPLE', b));
				cs.addColumn('blood.registrationDate', cs.asDate(b.registrationDate));
				cs.addColumn('blood.consent', cs.asConsent(b.consent, cs.asDate(b.consentDate)));
				cs.addColumn('blood.bloodSample', b.bloodSample);
				cs.addColumn('blood.samplingDate', cs.warnIfMissing(cs.asDateTime(b.samplingDate)));
				cs.addColumn('blood.freezerDate', cs.warnIfMissing(cs.asDateTime(b.freezerDate, b.samplingDate)));
				cs.addColumn('blood.serum', b.serum);
				cs.addColumn('blood.comment', b.comment);
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('blood.name', cs.asNoInfo('No blood information has been registered'));
				Doc.hide('blood-details');
			}
			else
			{
				Doc.hide('blood-info');
			}
		}

		var specimen = consentOk ? response.specimen : null;
		var noSpecimen = consentOk ? response.noSpecimen : null;
		var numSpecimenOrNoSpecimen = 0;
		if (specimen && specimen.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(specimen.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < specimen.length; i++)
			{
				var s = specimen[i];
				cs.addColumn('specimen.name', cs.asLink('SAMPLE', s));
				cs.addColumn('specimen.registrationDate', cs.asDate(s.registrationDate));
				cs.addColumn('specimen.storageBox', cs.asBioPlateLocation(s.bioWell));
				cs.addColumn('specimen.laterality', cs.warnIfMissing(s.laterality));
				cs.addColumn('specimen.samplingDate', cs.warnIfMissing(cs.asDateTime(s.samplingDate)));
				cs.addColumn('specimen.rnaLaterDate', cs.warnIfMissing(cs.asDateTime(s.rnaLaterDate, s.samplingDate)));
				cs.addColumn('specimen.pad', cs.warnIfMissing(s.pad));
				cs.addColumn('specimen.biopsyType', s.biopsyType);
				cs.addColumn('specimen.remainingQuantity', cs.asQuantity(s.remainingQuantity, ' mg', 1000));
				cs.addColumn('specimen.originalQuantity', cs.asQuantity(s.originalQuantity, ' mg', 1000));
				cs.addColumn('specimen.nofPieces', s.nofPieces);
				cs.addColumn('specimen.partitionComment', cs.truncate(s.partitionComment, truncateAt));
				cs.addColumn('specimen.comment', cs.truncate(s.comment, truncateAt));
				numSpecimenOrNoSpecimen++;
			}
			if (!noSpecimen || noSpecimen.length == 0)
			{
				Doc.hide('nospecimen-info');
			}
		}
		if (noSpecimen && noSpecimen.length > 0)
		{
			for (var i = 0; i < noSpecimen.length; i++)
			{
				var s = noSpecimen[i];
				cs.addColumn('nospecimen.name', cs.asLink('SAMPLE', s));
				cs.addColumn('nospecimen.registrationDate', cs.asDate(s.registrationDate));
				cs.addColumn('nospecimen.laterality', cs.warnIfMissing(s.laterality));
				cs.addColumn('nospecimen.samplingDate', cs.warnIfMissing(formatDate(s.samplingDate)));
				cs.addColumn('nospecimen.pad', cs.warnIfMissing(s.pad));
				cs.addColumn('nospecimen.reasonIfNoSpecimen', s.reasonIfNoSpecimen);
				cs.addColumn('nospecimen.comment', cs.truncate(s.comment, truncateAt));
				numSpecimenOrNoSpecimen++;
			}
			if (!specimen || specimen.length == 0)
			{
				cs.addColumn('specimen.name', cs.asNoInfo('No specimen received; see below for details'));
				Doc.hide('specimen-details');
			}
		}
		if (numSpecimenOrNoSpecimen == 0)
		{
			if (consentOk)
			{
				cs.addColumn('specimen.name', cs.asNoInfo('No specimen information has been registered'));
				Doc.hide('specimen-details');
			}
			else
			{
				Doc.hide('specimen-info');	
			}
			Doc.hide('nospecimen-info');
		}

		
		var histology = consentOk ? response.histology : null;
		if (histology && histology.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(histology.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < histology.length; i++)
			{
				var h = histology[i];
				cs.addColumn('histology.name', cs.asLink('SAMPLE', h));
				cs.addColumn('histology.registrationDate', cs.asDate(h.registrationDate));
				cs.addColumn('histology.partitionDate', cs.asDate(h.partitionDate));
				cs.addColumn('histology.usedQuantity', cs.asQuantity(h.originalQuantity, ' mg', 1000));

				var embedded = h.embedDate ? true : false;
				cs.addColumn('histology.embedded', embedded ? cs.asDate(h.embedDate) : 'No');
				cs.addColumn('histology.embedProtocol', embedded ? cs.asLink('PROTOCOL', h.embedProtocol, truncateAt) : null);
				cs.addColumn('histology.paraffinBlock', embedded ? cs.asBioPlateLocation(h.bioWell) : null);
				cs.addColumn('histology.storageBox', embedded ? null : cs.asBioPlateLocation(h.bioWell));

				var stained = h.stainDate ? true : false;
				var bestStain = stained ? h.bestStain : null;
				var scored = bestStain && bestStain.ScoreComplete != null ? true : false;
					
				cs.addColumn('histology.stained', stained ? cs.asDate(h.stainDate) : 'No');
				cs.addColumn('histology.nofSlides', stained ? h.numStains : null);
				cs.addColumn('histology.heGlass', stained ? cs.asBioPlateLocation(bestStain.bioWell) : null);
				cs.addColumn('histology.stainProtocol', stained ? cs.asLink('PROTOCOL', h.stainProtocol, truncateAt) : null);
				
				cs.addColumn('histology.scored', scored ? (bestStain.ScoreComplete ? 'Complete' : 'In progress') : 'No');
				cs.addColumn('histology.scoreInvasiveCancer', scored ? cs.asPercent(bestStain.ScoreInvasiveCancer) : null);
				cs.addColumn('histology.scoreInsituCancer', scored ? cs.asPercent(bestStain.ScoreInsituCancer) : null);
				cs.addColumn('histology.scoreLymphocytes', scored ? cs.asPercent(bestStain.ScoreLymphocytes) : null);
				cs.addColumn('histology.scoreNormal', scored ? cs.asPercent(bestStain.ScoreNormal) : null);
				cs.addColumn('histology.scoreStroma', scored ? cs.asPercent(bestStain.ScoreStroma) : null);
				cs.addColumn('histology.scoreFat', scored ? cs.asPercent(bestStain.ScoreFat) : null);
				cs.addColumn('histology.scoreComments', scored ? cs.truncate(bestStain.comments, truncateAt) : null);
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('histology.name', cs.asNoInfo('No histology information has been registered'));
				Doc.hide('histology-details');
			}
			else
			{
				Doc.hide('histology-info');
			}
		}

		var lysate = consentOk ? response.lysate : null;
		if (lysate && lysate.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(lysate.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < lysate.length; i++)
			{
				var lys = lysate[i];
				cs.addColumn('lysate.name', cs.asLink('EXTRACT', lys));
				cs.addColumn('lysate.registrationDate', cs.asDate(lys.registrationDate));
				cs.addColumn('lysate.partitionDate', cs.asDate(lys.partitionDate));
				cs.addColumn('lysate.usedQuantity', cs.asQuantity(lys.usedQuantity, ' mg', 1000));
				cs.addColumn('lysate.multiplePieces', lys.multiplePieces ? 'Yes' : 'No');
				cs.addColumn('lysate.comment', cs.truncate(lys.comment, truncateAt));
				
				var lysis = lys.lysisDate ? true : false;
				cs.addColumn('lysate.lysisDate', lysis ? cs.asDate(lys.lysisDate) : 'No');
				cs.addColumn('lysate.storageBox', lysis ? cs.asBioPlateLocation(lys.bioWell) : null);
				cs.addColumn('lysate.lysisProtocol', lysis ? cs.asLink('PROTOCOL', lys.lysisProtocol, truncateAt) : null);
				cs.addColumn('lysate.remainingQuantity', lysis ? cs.asQuantity(lys.remainingQuantity, ' µg') : null);
				cs.addColumn('lysate.originalQuantity', lysis ? cs.asQuantity(lys.originalQuantity, ' µg') : null);
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('lysate.name', cs.asNoInfo('No lysate information has been registered'));
				Doc.hide('lysate-details');
			}
			else
			{
				Doc.hide('lysate-info');
			}
		}

		var rna = consentOk ? response.rna : null;
		if (rna && rna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(rna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < rna.length; i++)
			{
				var r = rna[i];
				cs.addColumn('rna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('rna.flag', r.flag ? cs.asFlagInfo(r.flag, 'flag.png') : null);
				cs.addColumn('rna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('rna.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('rna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
				cs.addColumn('rna.extractionDate', cs.asDate(r.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.extractionDate, 'RNA extraction: ' + r.name, 'thermometer.png'));
				cs.addColumn('rna.extractionProtocol', cs.asLink('PROTOCOL', r.extractionProtocol, truncateAt));
				cs.addColumn('rna.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' µg'));
				cs.addColumn('rna.originalQuantity', cs.asQuantity(r.originalQuantity, ' µg'));
				cs.addColumn('rna.ndConc', cs.asQuantity(r.ndConc, ' ng/µl'));
				cs.addColumn('rna.comment', cs.truncate(r.comment, truncateAt));
				
				var hasQc = r.nofQc ? true : false;
				cs.addColumn('rna.qc', hasQc ? (r.qcDate ? cs.asDate(r.qcDate) : cs.asNoInfo('Waiting for Caliper...')) : 'No');
				cs.addColumn('rna.nofQc', hasQc ? r.nofQc : null);
				cs.addColumn('rna.qcProtocol', hasQc ? cs.asLink('PROTOCOL', r.qcProtocol, truncateAt) : null);
				cs.addColumn('rna.qcPlate', hasQc ? cs.asBioPlateLocation(r.qcPlate)+cs.asFileLink(r.qcPdf, 'pdffile.png') : null);
				cs.addColumn('rna.qcRqs', hasQc && r.qcRqs ? formatNumber(r.qcRqs, null, 1) : null);
				cs.addColumn('rna.qcRin', hasQc && r.qcRin ? formatNumber(r.qcRin, null, 1) : null);
				cs.addColumn('rna.qcComment', hasQc ? cs.truncate(r.qcComment, truncateAt) : null);
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('rna.name', cs.asNoInfo('No RNA information has been registered'));
				Doc.hide('rna-details');
			}
			else
			{
				Doc.hide('rna-info');
			}
		}

		var dna = consentOk ? response.dna : null;
		if (dna && dna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(dna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < dna.length; i++)
			{
				var d = dna[i];
				cs.addColumn('dna.name', cs.asLink('EXTRACT', d));
				cs.addColumn('dna.registrationDate', cs.asDate(d.registrationDate));
				cs.addColumn('dna.storageBox', cs.asBioPlateLocation(d.bioWell));
				cs.addColumn('dna.extractionDate', cs.asDate(d.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', d.extractionDate, 'DNA extraction: ' + d.name, 'thermometer.png'));
				cs.addColumn('dna.extractionProtocol', cs.asLink('PROTOCOL', d.extractionProtocol, truncateAt));
				cs.addColumn('dna.remainingQuantity', cs.asQuantity(d.remainingQuantity, ' µg'));
				cs.addColumn('dna.originalQuantity', cs.asQuantity(d.originalQuantity, ' µg'));
				cs.addColumn('dna.ndConc', cs.asQuantity(d.ndConc, ' ng/µl'));
				cs.addColumn('dna.comment', cs.truncate(d.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('dna.name', cs.asNoInfo('No DNA information has been registered'));
				Doc.hide('dna-details');
			}
			else
			{
				Doc.hide('dna-info');
			}
		}

		var ft = consentOk ? response.flowThrough : null;
		if (ft && ft.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(ft.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < ft.length; i++)
			{
				var f = ft[i];
				cs.addColumn('ft.name', cs.asLink('EXTRACT', f));
				cs.addColumn('ft.registrationDate', cs.asDate(f.registrationDate));
				cs.addColumn('ft.storageBox', cs.asBioPlateLocation(f.bioWell));
				cs.addColumn('ft.extractionDate', cs.asDate(f.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', f.extractionDate, 'FlowThrough extraction: ' + f.name, 'thermometer.png'));
				cs.addColumn('ft.extractionProtocol', cs.asLink('PROTOCOL', f.extractionProtocol, truncateAt));
				cs.addColumn('ft.remainingQuantity', cs.asQuantity(f.remainingQuantity, ' µg'));
				cs.addColumn('ft.originalQuantity', cs.asQuantity(f.originalQuantity, ' µg'));
				cs.addColumn('ft.comment', cs.truncate(f.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('ft.name', cs.asNoInfo('No FlowThrough information has been registered'));
				Doc.hide('ft-details');
			}
			else
			{
				Doc.hide('ft-info');
			}
		}

		var mrna = consentOk ? response.mrna : null;
		if (mrna && mrna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(mrna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < mrna.length; i++)
			{
				var r = mrna[i];
				cs.addColumn('mrna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('mrna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('mrna.storageBox', cs.asBioPlateLocation(r.bioWell)+cs.asFileLink(r.platePdf, 'pdffile.png'));
				cs.addColumn('mrna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('mrna.cleanupDate', cs.asDate(r.cleanupDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.cleanupDate, 'mRNA cleanup: ' + r.name, 'thermometer.png'));
				cs.addColumn('mrna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
				cs.addColumn('mrna.comment', cs.truncate(r.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('mrna.name', cs.asNoInfo('No mRNA information has been registered'));
				Doc.hide('mrna-details');
			}
			else
			{
				Doc.hide('mrna-info');
			}
		}

		var cdna = consentOk ? response.cdna : null;
		if (cdna && cdna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(cdna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < cdna.length; i++)
			{
				var r = cdna[i];
				cs.addColumn('cdna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('cdna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('cdna.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('cdna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('cdna.synthesisDate', cs.asDate(r.synthesisDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.synthesisDate, 'cDNA synthesis: ' + r.name, 'thermometer.png'));
				cs.addColumn('cdna.comment', cs.truncate(r.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('cdna.name', cs.asNoInfo('No cDNA information has been registered'));
				Doc.hide('cdna-details');
			}
			else
			{
				Doc.hide('cdna-info');
			}
		}
		
		var lib = consentOk ? response.lib : null;
		if (lib && lib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(lib.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < lib.length; i++)
			{
				var r = lib[i];
				cs.addColumn('lib.name', cs.asLink('EXTRACT', r));
				cs.addColumn('lib.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('lib.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('lib.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('lib.cleanupDate', cs.asDate(r.cleanupDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.cleanupDate, 'Library cleanup: ' + r.name, 'thermometer.png'));
				cs.addColumn('lib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
				cs.addColumn('lib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
				cs.addColumn('lib.barcode.name', r.barcode.name);
				cs.addColumn('lib.ca_size', cs.asQuantity(r.ca_size, ''));
				cs.addColumn('lib.molarity', cs.asQuantity(r.library_molarity_est, ' nM'));
				cs.addColumn('lib.adapterFrac', cs.asPercent(r.library_frac_adpt));
				cs.addColumn('lib.qubitconc', cs.asQuantity(r.qubitconc, ' ng/µl'));
				cs.addColumn('lib.comment', cs.truncate(r.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('lib.name', cs.asNoInfo('No library information has been registered'));
				Doc.hide('lib-details');
			}
			else
			{
				Doc.hide('lib-info');
			}
		}

		var pooledLib = consentOk ? response.pooledlib : null;
		if (pooledLib && pooledLib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(pooledLib.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < pooledLib.length; i++)
			{
				var r = pooledLib[i];
				cs.addColumn('pooledlib.name', cs.asLink('EXTRACT', r));
				cs.addColumn('pooledlib.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('pooledlib.poolDate', cs.asDate(r.poolDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.poolDate, 'Pooled library pooling: ' + r.name, 'thermometer.png'));
				cs.addColumn('pooledlib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
				cs.addColumn('pooledlib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
				cs.addColumn('pooledlib.poolMolarity', cs.asQuantity(r.poolMolarity, ' nM'));
				cs.addColumn('pooledlib.poolConc', cs.asQuantity(r.poolConc, ' ng/µl'));
				cs.addColumn('pooledlib.comment', cs.truncate(r.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('pooledlib.name', cs.asNoInfo('No pooled library information has been registered'));
				Doc.hide('pooledlib-details');
			}
			else
			{
				Doc.hide('pooledlib-info');
			}
		}

		var sequencingRuns = consentOk ? response.sequencingRun : null;
		if (sequencingRuns && sequencingRuns.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(sequencingRuns.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < sequencingRuns.length; i++)
			{
				var sr = sequencingRuns[i];
				var fc = sr.flowCell;
				cs.addColumn('sequencingRun.name', cs.asLink('DERIVEDBIOASSAY', sr));
				cs.addColumn('sequencingRun.flowCell', cs.asLink('PHYSICALBIOASSAY', fc));
				cs.addColumn('sequencingRun.result', (sr.result && sr.result != 'Successful') ? cs.asFailInfo(sr.result, 'error.png') : null);
				cs.addColumn('sequencingRun.clusterDate', cs.asDate(fc.clusterDate));
				cs.addColumn('sequencingRun.startDate', cs.asDate(sr.startDate));
				cs.addColumn('sequencingRun.endDate', cs.asDate(sr.endDate));
				cs.addColumn('sequencingRun.comment', cs.truncate(sr.comment, truncateAt));
			}
		}
		else
		{
			if (consentOk)
			{
				cs.addColumn('sequencingRun.name', cs.asNoInfo('No sequencing information has been registered'));
				Doc.hide('sequencingRun-details');
			}
			else
			{
				Doc.hide('sequencingRun-info');
			}
		}

		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}
		
		var linkedFiles = document.getElementsByClassName('linked-file');
		for (var fileNo = 0; fileNo < linkedFiles.length; fileNo++)
		{
			Events.addEventHandler(linkedFiles[fileNo], 'click', Files.viewFileOnClick);
		}
		
		var linkedLabEnvs = document.getElementsByClassName('linked-labenv');
		for (var labEnvNo = 0; labEnvNo < linkedLabEnvs.length; labEnvNo++)
		{
			Events.addEventHandler(linkedLabEnvs[labEnvNo], 'click', LabEnvs.viewDataOnClick);
		}
		
		var linkedCases = document.getElementsByClassName('linked-case');
		for (var caseNo = 0; caseNo < linkedCases.length; caseNo++)
		{
			Events.addEventHandler(linkedCases[caseNo], 'click', cs.caseSummary);
		}
	}
	
	cs.initSections = function(sections)
	{
		for (var s = 0; s < sections.length; s++)
		{
			var section = sections[s];

			var details = Doc.element(section.id+'-details');
			// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
			if (details && details.style.display != 'none')
			{
				var headerDiv = Doc.element(section.id+'-header');
				if (headerDiv)
				{
					Doc.addClass(headerDiv, 'link');
					headerDiv.title = 'Hide/show details';
					headerDiv.sectionId = section.id;
					Events.addEventHandler(headerDiv, 'click', cs.toggleDetails);
				}
				
				var sectionDiv = Doc.element(section.id+'-info');
				Doc.addOrRemoveClass(sectionDiv, 'hide-details', section.hiddenDetails);
			}
		}
	}

	cs.toggleDetails = function(event)
	{
		var sectionId = event.currentTarget.sectionId;
		var details = Doc.element(sectionId+'-details');
		// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
		if (details.style.display == 'none') return;
		
		// Add 'hide-details' class to main <div section-info> if it is visible
		var section = Doc.element(sectionId+'-info');
		var isVisible = section.className.indexOf('hide-details') == -1;
		Doc.addOrRemoveClass(section, 'hide-details', isVisible);

		// Save to BASE setting
		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=SetSectionVisibility';
		url += '&section='+sectionId+'&hidden='+(isVisible ? 1 : 0);
		var request = Ajax.getXmlHttpRequest();
		request.open("POST", url, false); 
		request.send(null);
		if (debug) App.debug(request.responseText);
	}

	cs.addColumn = function(id, value)
	{
		var tr = Doc.element(id);
		var td = document.createElement('td');
		if (tr.hasAddedColumns)
		{
			td.className = 'extra-column';
			tr.childNodes[tr.childNodes.length-1].className += ' fixed-column';
		}
		tr.hasAddedColumns = true;
		if (value != null)
		{
			td.innerHTML = value == '' ? '&nbsp;' : value;
			if (tr.className.indexOf('dynamic-column') >= 0)
			{
				tr.style.display = 'table-row';
			}
		}
		else
		{
			td.innerHTML = '&nbsp;';
		}
		tr.appendChild(td);
	}

	cs.asLink = function(itemType, item, maxLength)
	{
		var link = '';
		if (item)
		{
			var name = cs.truncate(item.name, maxLength);
			if (item.id)
			{
				link = '<span class="link linked-item" data-item-type="'+itemType+'" data-item-id="'+item.id+'" data-no-edit="'+(item.editable ? 0 : 1)+'">';
				link += name + '</span>';
			}
			else
			{
				link = name;
			}
		}
		return link;
	}
	
	cs.asFileLink = function(file, icon)
	{
		var link = '';
		if (file)
		{
			link = '<span class="link linked-file" data-file-id="'+file.id+'"><img src="../images/'+icon+'"></span>';
		}
		return link;
	}

	cs.asLabEnvLink = function(sensor, date, title, icon)
	{
		var link = '';
		// Check if BASE extension LabEnv is installed
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		if (homeLabEnv)
		{
			if (date)
			{
				if (sensor)
				{
					// Link to pop-up window with lab environment data for the chosen sensor and date
					link = '<span class="link linked-labenv" data-sensor="'+sensor+'" data-date="'+date+'" data-title="'+title+'"><img src="../images/'+icon+'"></span>';
				}
			}
		}
		return link;
	}

	cs.asCaseList = function(allCases, mainCase)
	{
		var html = '';
		for (var i = 0; i < allCases.length; i++)
		{
			var cse = allCases[i];
			if (html != '') html += ', ';
			if (cse != mainCase)
			{
				html += '<span class="link linked-case" data-case-name="'+cse+'" title="Show summary of case #'+cse+'">'+cse+'</span>';
			}
			else
			{
				html += '<b>' + cse + '</b>';
			}
		}
		return allCases.length > 1 ? html : null;
	}

	
	// Format value as a date
	cs.asDate = function(value)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = value.substr(0, 4) + '-' + value.substr(4, 2) + '-' + value.substr(6, 2);
		}
		return value;
	}

	// Format as date+time value
	cs.asDateTime = function(value, compareToDate)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = cs.asDate(value);
		}
		else if (value.length == 13)
		{
			// If the compareToDate is the same day as the 'value' date, skip the date and replace with white-space
			if (compareToDate && value.substr(0, 8) == compareToDate.substr(0, 8))
			{
				value = '<span class="invisible">'+cs.asDate(value.substr(0, 8)) + '</span> ' + value.substr(9, 2) + ':'+value.substr(11, 2)
			}
			else
			{
				value = cs.asDate(value.substr(0, 8)) + ' ' + value.substr(9, 2) + ':'+value.substr(11, 2);
			}
		}
		return value;
	}

	// Style the value as a quantity with (optional) unit
	cs.asQuantity = function(value, unit, scale)
	{
		var result = '';
		if (value != null)
		{
			if (scale) value = value / scale;
			result = formatNumber(value, unit, 2);
		}
		return result;
	}
	
	// Style the value as a percentage value
	cs.asPercent = function(value)
	{
		var result = '';
		if (value != null)
		{
			result = value + '%';
		}
		return result;
	}


	// Style the message as a warning message
	cs.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	cs.warnIfMissing = function(value)
	{
		var message = value ? value : cs.asWarning('Missing');
		return message;
	}
	
	// Style the message as a 'no information' message
	cs.asNoInfo = function(message)
	{
		message = '<span class="no-info">' + message + '</span>';
		return message;
	}
	
	// Style the message as a 'flag information' message
	cs.asFlagInfo = function(message, icon)
	{
		message = '<span class="flag-info">' + message + '&nbsp;' + '<img src="../images/'+icon+'"></span>';
		return message;
	}

	// Style the message as a 'fail information' message
	cs.asFailInfo = function(message, icon)
	{
		message = '<span class="fail-info">' + message + '&nbsp;' + '<img src="../images/'+icon+'"></span>';
		return message;
	}
	
	cs.asConsent = function(consent, consentDate)
	{
		var warn = !consent || !consentDate;
		var message;
		if (!consent)
		{
			message = 'Missing' + (consentDate ? ', ' + consentDate : '');
		}
		else
		{
			message = consent + ', ' + (consentDate ? consentDate : 'Missing date');
			
		}
		return warn ? cs.asWarning(message) : message;
	}
	
	// A biowell as the location
	cs.asBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = cs.asLink('BIOPLATE', plate)  + ' ' + well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = storage.name;
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;

			text = '<span class="more-info" title="' + tmp.join('; ')+'">' + text + ' </span>';
		}
		return text;
	}

	cs.truncate = function(value, maxLength)
	{
		if (!value) return value;
		if (maxLength > 2 && value.length > maxLength) 
		{
			var tmp = '<span class="truncated" title="'+value+'">'+value.substring(0, maxLength-2) + '...</span>';
			value = tmp;
		}
		return value;
	}

	cs.showConfidentialOnChange = function()
	{
		var frm = document.forms['reggie'];
		Doc.addOrRemoveClass('all-info', 'hide-confidential', !frm.showConfidential.checked);
	}

	cs.caseSummary = function(event)
	{
		var frm = document.forms['reggie'];
		var caseName = Data.get(event.currentTarget, 'case-name');
		
		if (!caseName)
		{
			caseName = frm.caseName.value;
			if (!caseName)
			{
				Forms.showNotification('caseName', 'Please enter a 7-digit case id', null, null, 'left');
				return;
			}
		}
		var url = 'case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&pageType='+Data.get('page-data', 'page-type');
		location.href = url;
	}

	cs.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		openPrintWindow(App.getSessionId(), 'all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	return cs;
}();

var LabEnvs = function()
{
	var labenvs = {};

	/**
		Event handler that calls 'labenv.jar/reports/labenvironmentdatabaseeventpopup.jsp'
		when clicking on the attached target. The sensor id should be stored
		in attribute 'data-sensor', the date in 'data-date',
		and the title in 'data-title'.
	*/
	labenvs.viewDataOnClick = function(event)
	{
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		var title = Data.get(event.currentTarget, 'title');
		var sensor = Data.get(event.currentTarget, 'sensor');
		var date = Data.get(event.currentTarget, 'date');
		var url = homeLabEnv;
		url += '/reports/labenvironmentdatabaseeventpopup.jsp';
		url += '?ID='+App.getSessionId();
		url += '&pageTitle='+encodeURIComponent(title);
		url += '&labSensorUrl='+encodeURIComponent(sensor);
		url += '&originDate='+encodeURIComponent(date);
		Dialogs.openPopup(url, title, 820, 768);
	}
	
	return labenvs;
}();

Doc.onLoad(CaseSummary.initPage);
